@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2012 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -38,15 +38,16 @@
 #include <gccv/group.h>
 #include <gccv/text.h>
 #include <gccv/text-tag.h>
+#include <gcu/chain.h>
 #include <gcu/element.h>
-#include <openbabel/mol.h>
+#include <gcu/objprops.h>
+#include <gcugtk/ui-manager.h>
 #include <glib/gi18n-lib.h>
 #include <cstdlib>
 #include <cstring>
 #include <sstream>
 
 using namespace gcu;
-using namespace OpenBabel;
 using namespace std;
 
 #define ATOM_EPSILON 0.1
@@ -126,38 +127,6 @@ Atom::Atom (int Z, double x, double y, double z):
 	SetZ(Z);
 }
 
-Atom::Atom (OBAtom* atom):
-	gcu::Atom (),
-	ItemClient (),
-	m_ShowSymbol (false),
-	m_HPosStyle (AUTO_HPOS)
-{
-	m_x = atom->GetX ();
-	m_y = - atom->GetY ();
-	m_z = atom->GetZ ();
-	m_nlp = m_nlu = 0;
-	gchar* Id = g_strdup_printf ("a%d", atom->GetIdx());
-	SetId (Id);
-	g_free (Id);
-	m_HPos = GetBestSide ();
-	m_ascent = 0;
-	m_CHeight = 0.;
-	m_Changed = 0;
-	m_AvailPosCached = false;
-	m_OccupiedPos = 0;
-	m_ChargePos = 0xff;
-	m_ChargeAngle = 0.;
-	m_ChargeDist = 0.;
-	m_ChargeAutoPos = true;
-	m_Layout = m_ChargeLayout = m_HLayout = NULL;
-	m_DrawCircle = false;
-	m_Charge = atom->GetFormalCharge ();
-	m_SWidth = 0.;
-	m_ChargeItem = NULL;
-	m_ShowCharge = true;
-	SetZ (atom->GetAtomicNum ());
-}
-
 void Atom::SetZ (int Z)
 {
 	gcu::Atom::SetZ (Z);
@@ -263,14 +232,20 @@ void Atom::Update ()
 	}
 	int nbonds = GetTotalBondsNumber ();
 	if (m_Valence > 0 && !m_Element->IsMetallic ()) {
-		m_nlp = (m_Element->GetValenceElectrons () - m_Valence) / 2;
+		m_nlp = (m_Element->GetValenceElectrons () - ((nbonds > m_Valence)? nbonds: m_Valence)) / 2;
 		if ((m_Charge > 0) && (m_nlp > 0)) m_nlp -= (m_Charge + 1) / 2;
 		else if (m_Charge < 0)
 			m_nlp -= m_Charge;
 		if (m_nlp < nexplp) // Can this occur ?
 			m_nlp = nexplp;
-		else if (m_nlp > m_ValenceOrbitals - nbonds - nexplu)
-			m_nlp = m_ValenceOrbitals - nbonds - nexplu;
+		else if (m_nlp > m_ValenceOrbitals - nbonds - nexplu) {
+			int max_bonds = Element::GetElement (m_Z)->GetMaxBonds ();
+			if (max_bonds > m_ValenceOrbitals) {
+				if (m_nlp > max_bonds - nbonds - nexplu)
+					m_nlp = max_bonds - nbonds - nexplu;// FIXME, might be wrong, but will this occur?
+			} else
+				m_nlp = m_ValenceOrbitals - nbonds - nexplu;
+		}
 		if (m_nlp < 0)
 			m_nlp = 0;
 		nb = m_Element->GetValenceElectrons () - 2 * m_nlp - m_Charge;
@@ -431,6 +406,8 @@ gccv::Anchor Atom::GetChargePosition (unsigned char& Pos, double Angle, double&
 	double angle;
 	if (m_ChargePos != 0xff)
 		m_OccupiedPos &= ~m_ChargePos;
+	if (m_ChargeAutoPos) // reset the charge position
+		m_ChargePos = 0xff;
 	if (!m_AvailPosCached)
 		UpdateAvailablePositions ();
 	if (m_ChargePos != 0xff)
@@ -477,7 +454,7 @@ gccv::Anchor Atom::GetChargePosition (unsigned char& Pos, double Angle, double&
 				}
 				angle = *n;
 			}
-		} 
+		}
 	} else if (Pos) {
 		if (!(Pos & m_AvailPos) && (Pos != m_ChargePos))
 			return gccv::AnchorCenter;
@@ -554,7 +531,7 @@ gccv::Anchor Atom::GetChargePosition (unsigned char& Pos, double Angle, double&
 				y = m_y - m_width / 2. * t;
 				return gccv::AnchorWest;
 			}
-		}			
+		}
 	}
 	return gccv::AnchorCenter; // should not occur
 }
@@ -640,22 +617,34 @@ bool Atom::LoadNode (xmlNodePtr)
 
 void Atom::SetSelected (int state)
 {
-	GOColor textcolor, othercolor;
-	
-	switch (state) {	
+	GOColor textcolor, othercolor, chargecolor;
+
+	switch (state) {
 	default:
-	case SelStateUnselected:
+	case SelStateUnselected: {
 		textcolor = 0;
-		othercolor = GO_COLOR_BLACK;
+		if (static_cast < Document * > (GetDocument ())->GetUseAtomColors ()) {
+			double *c = Element::GetElement (GetZ ())->GetDefaultColor ();
+			int r = c[0] * 255, g = c[1] * 255, b = c[2] * 255;
+			if (r > 200 && g > 200 && b > 200) {
+				r = 255 - r;
+				g = 255 - g;
+				b = 255 - b;
+			}
+			othercolor = GO_COLOR_FROM_RGB (r, g, b);
+		} else
+			othercolor = GO_COLOR_BLACK;
+		chargecolor = GO_COLOR_BLACK;
 		break;
+	}
 	case SelStateSelected:
-		othercolor = textcolor = SelectColor;
+		chargecolor = othercolor = textcolor = SelectColor;
 		break;
 	case SelStateUpdating:
-		othercolor = textcolor = AddColor;
+		chargecolor = othercolor = textcolor = AddColor;
 		break;
 	case SelStateErasing:
-		othercolor = textcolor = DeleteColor;
+		chargecolor = othercolor = textcolor = DeleteColor;
 		break;
 	}
 	gccv::Group *group = static_cast <gccv::Group *> (m_Item);
@@ -665,9 +654,9 @@ void Atom::SetSelected (int state)
 		if (item->GetClient () == this) {
 			gccv::FillItem *fill;
 			gccv::Text *text;
-			if ((text = dynamic_cast <gccv::Text *> (item)))
-				text->SetColor (othercolor);
-			else if ((fill = dynamic_cast <gccv::Rectangle *> (item)))
+			if ((text = dynamic_cast <gccv::Text *> (item))) {
+				text->SetColor (g_ascii_isalpha (*text->GetText ())?othercolor: chargecolor);
+			} else if ((fill = dynamic_cast <gccv::Rectangle *> (item)))
 				fill->SetFillColor (textcolor);
 			else if ((fill = dynamic_cast <gccv::FillItem *> (item)))
 				fill->SetFillColor (othercolor);
@@ -711,7 +700,7 @@ void Atom::AddToMolecule (Molecule* Mol)
 {
 	Mol->AddAtom (this);
 }
-	
+
 double Atom::GetYAlign ()
 {
 	return m_y;
@@ -733,7 +722,7 @@ bool Atom::HasImplicitElectronPairs ()
 		return (m_nlp > nexplp);
 	}
 	unsigned nel = 0;
-	while (electron) { 
+	while (electron) {
 		electron = dynamic_cast <Electron *> (obj);
 		if (electron) {
 			if (electron->IsPair ())
@@ -753,7 +742,7 @@ bool Atom::MayHaveImplicitUnpairedElectrons ()
 	map<string, Object*>::iterator i;
 	Electron* electron = (Electron*) GetFirstChild (i);
 	unsigned nel = 0;
-	while (electron) { 
+	while (electron) {
 		if (electron->IsPair ())
 			nel += 2;
 		else
@@ -799,7 +788,7 @@ bool Atom::GetRelativePosition (double angle, double& x, double& y)
 		} else {
 			x = m_width / 2.;
 			y = -m_width / 2. * t;
-		}			
+		}
 		return true;
 	}
 	return false;
@@ -840,21 +829,12 @@ void Atom::NotifyPositionOccupation (unsigned char pos, bool occupied)
 	else
 		m_OccupiedPos &= ~pos;
 }
-	
+
 xmlNodePtr Atom::Save (xmlDocPtr xml) const
 {
-	xmlNodePtr node = gcu::Atom::Save (xml), child;
-	if (node) {
-	// Save electrons
-		map<string, Object*>::const_iterator i;
-		Electron* electron = (Electron*) GetFirstChild (i);
-		while (electron){ 
-			child = electron->Save (xml);
-			if (child)
-				xmlAddChild (node, child);
-			electron = (Electron*) GetNextChild (i);
-		}
-	}
+	xmlNodePtr node = gcu::Atom::Save (xml);
+	if (node)
+		SaveChildren (xml, node);
 	if (m_Charge && !m_ChargeAutoPos) {
 		char *buf;
 		if (m_ChargePos) {
@@ -927,7 +907,7 @@ xmlNodePtr Atom::Save (xmlDocPtr xml) const
 					reinterpret_cast <xmlChar const*> ("no"));
 	return node;
 }
-	
+
 bool Atom::Load (xmlNodePtr node)
 {
 	if (!gcu::Atom::Load (node))
@@ -1031,12 +1011,12 @@ bool Atom::Load (xmlNodePtr node)
 	}
 	return true;
 }
-	
+
 bool Atom::AcceptCharge (int charge) {
 	unsigned nb = GetTotalBondsNumber (), ne = 0;
 	map<string, Object*>::iterator i;
 	Electron* electron = (Electron*) GetFirstChild (i);
-	while (electron){ 
+	while (electron){
 		if (electron->IsPair ())
 			ne += 2;
 		else
@@ -1149,7 +1129,7 @@ static void do_display_symbol (GtkToggleAction *action, Atom *pAtom)
 		view->Update (bond);
 		bond = static_cast <Bond *> (pAtom->GetNextBond (i));
 	}
-	
+
 }
 
 static void do_choose_H_pos (Atom* Atom)
@@ -1168,7 +1148,7 @@ static void do_show_charge (GtkToggleAction *action, Atom *atom)
 	Doc->FinishOperation ();
 	View *view = Doc->GetView ();
 	view->Update (atom);
-	
+
 }
 
 static void do_select_child (Object *obj)
@@ -1195,9 +1175,10 @@ static void do_child_properties (Object *obj)
 	obj->ShowPropertiesDialog ();
 }
 
-bool Atom::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y)
+bool Atom::BuildContextualMenu (gcu::UIManager *UIManager, Object *object, double x, double y)
 {
 	bool result = false;
+	GtkUIManager *uim = static_cast < gcugtk::UIManager * > (UIManager)->GetUIManager ();
 	GtkActionGroup *group = NULL;
 	GtkAction *action;
 	if (GetZ () == 6 && GetBondsNumber() != 0) {
@@ -1210,7 +1191,7 @@ bool Atom::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double
 		g_signal_connect (action, "toggled", G_CALLBACK (do_display_symbol), this);
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
-		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Atom'><menuitem action='show-symbol'/></menu></popup></ui>", -1, NULL);
+		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Atom'><menuitem action='show-symbol'/></menu></popup></ui>", -1, NULL);
 		result = true;
 	}
 	if (m_nH) {
@@ -1224,7 +1205,7 @@ bool Atom::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_choose_H_pos), this);
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
-		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Atom'><menuitem action='H-position'/></menu></popup></ui>", -1, NULL);
+		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Atom'><menuitem action='H-position'/></menu></popup></ui>", -1, NULL);
 	}
 	if (m_Charge) {
 		if (!group) {
@@ -1238,7 +1219,7 @@ bool Atom::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double
 		g_signal_connect (action, "toggled", G_CALLBACK (do_show_charge), this);
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
-		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Atom'><menuitem action='show-charge'/></menu></popup></ui>", -1, NULL);
+		gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Atom'><menuitem action='show-charge'/></menu></popup></ui>", -1, NULL);
     }
 	if (object == this && HasChildren ()) {
 		if (!group)
@@ -1276,11 +1257,11 @@ bool Atom::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double
 			if (has_props)
 				str << "'/><menuitem action='" << prop;
 			str << "'/></menu></popup></ui>";
-			gtk_ui_manager_add_ui_from_string (UIManager, str.str ().c_str (), -1, NULL);
+			gtk_ui_manager_add_ui_from_string (uim, str.str ().c_str (), -1, NULL);
 		}
 	}
 	if (group) {
-		gtk_ui_manager_insert_action_group (UIManager, group, 0);
+		gtk_ui_manager_insert_action_group (uim, group, 0);
 		g_object_unref (group);
 	}
 	return result | Object::BuildContextualMenu (UIManager, object, x, y);
@@ -1329,6 +1310,18 @@ void Atom::AddItem ()
 	View *view = doc->GetView ();
 	Theme *theme = doc->GetTheme ();
 	double x, y;
+	GOColor textcolor;
+	if (doc->GetUseAtomColors ()) {
+		double *c = Element::GetElement (GetZ ())->GetDefaultColor ();
+		int r = c[0] * 255, g = c[1] * 255, b = c[2] * 255;
+		if (r > 200 && g > 200 && b > 200) {
+			r = 255 - r;
+			g = 255 - g;
+			b = 255 - b;
+		}
+		textcolor = GO_COLOR_FROM_RGB (r, g, b);
+	} else
+		textcolor = GO_COLOR_BLACK;
 	GetCoords (&x, &y);
 	x *= theme->GetZoomFactor ();
 	y *= theme->GetZoomFactor ();
@@ -1337,7 +1330,7 @@ void Atom::AddItem ()
 	view->GetCanvas ()->GetRoot ()->MoveToFront (group);
 	if ((GetZ() != 6) || (GetBondsNumber() == 0) || m_ShowSymbol) {
 		gccv::Text *text = new gccv::Text (group, 0., 0., this);
-		text->SetColor ((view->GetData ()->IsSelected (this))? SelectColor: GO_COLOR_BLACK);
+		text->SetColor ((view->GetData ()->IsSelected (this))? SelectColor: textcolor);
 		text->SetPadding (theme->GetPadding ());
 		text->SetLineColor (0);
 		text->SetLineWidth (0.);
@@ -1349,7 +1342,7 @@ void Atom::AddItem ()
 		int n = GetAttachedHydrogens ();
 		gccv::Rect ink, logical;
 		text->GetBounds (&ink, &logical);
-		BuildSymbolGeometry (text->GetWidth (), ink.y1 - ink.y0, /*text->GetAscent () - view->GetCHeight ()*/ - ink.y0);		
+		BuildSymbolGeometry (text->GetWidth (), ink.y1 - ink.y0, /*text->GetAscent () - view->GetCHeight ()*/ - ink.y0);
 		m_width = (ink.x1 - ink.x0 + 2 * theme->GetPadding ()) / theme->GetZoomFactor ();
 		m_height = (ink.y1 - ink.y0 + 2 * theme->GetPadding ()) / theme->GetZoomFactor ();
 		if (n > 0) {
@@ -1360,7 +1353,7 @@ void Atom::AddItem ()
 				g_free (str);
 			}
 			text = new gccv::Text (group, 0., 0., this);
-			text->SetColor ((view->GetData ()->IsSelected (this))? SelectColor: GO_COLOR_BLACK);
+			text->SetColor ((view->GetData ()->IsSelected (this))? SelectColor: textcolor);
 			text->SetPadding (theme->GetPadding ());
 			text->SetLineColor (0);
 			text->SetLineWidth (0.);
@@ -1462,7 +1455,7 @@ bool Atom::HasAvailableElectrons (bool paired)
 	if (paired) {
 		if (m_nlp)
 			return true;
-		while (obj) { 
+		while (obj) {
 			if (electron && electron->IsPair ())
 				return true;
 			obj = GetNextChild (i);
@@ -1493,4 +1486,163 @@ Bond *Atom::GetBondAtAngle (double angle)
 	return res;
 }
 
+bool Atom::SetProperty (unsigned property, char const *value)
+{
+	switch (property) {
+	case GCU_PROP_ATOM_PARITY: {
+		std::istringstream iss (value);
+		int p;
+		std::string a0, a1, a2, a3;
+		iss >> p >> a0 >> a1 >> a2 >> a3;
+		if (p == 0)
+			return false;
+		gcu::Document *doc = GetDocument ();
+		doc->SetTarget (a0.c_str (), reinterpret_cast <Object **> (m_Bonded + ((p > 0)? 0: 1)), GetParent (), this);
+		doc->SetTarget (a1.c_str (), reinterpret_cast <Object **> (m_Bonded + ((p > 0)? 1: 0)), GetParent (), this);
+		doc->SetTarget (a2.c_str (), reinterpret_cast <Object **> (m_Bonded + 2), GetParent (), this);
+		if (a3.length () == 0)
+			m_Bonded[3] = NULL;
+		else
+			doc->SetTarget (a3.c_str (), reinterpret_cast <Object **> (m_Bonded + 3), GetParent (), this);
+		static_cast < Molecule * > (GetMolecule ())->AddChiralAtom (this);
+		break;
+	}
+	default:
+		return gcu::Atom::SetProperty (property, value);
+	}
+	return  true;
+}
+
+bool Atom::UpdateStereoBonds ()
+{
+	unsigned length[4]; // lengths before end or cycle 0 means cyclic bond
+	unsigned cycle_size[4]; // size of the nearest and largest attached cycle
+	unsigned cycle_pos[4]; // position of the cycle
+	Bond *bond[4];
+	double x[4], y[4];
+	std::list < unsigned > sorted;
+	std::list < unsigned >::iterator s, send;
+	// we need to determine wich bonds should be considered as stereobonds
+	// the answer is that stereocenters should not be bonded by a stereobond
+	// then the bond should not be cyclic
+	// then it shoud be in the shortest possible chain
+	// it should be not more that 90° from the nearest bond (which might also be
+	// a stereobond)
+	for (unsigned i = 0; i < 4; i++) {
+		if (!m_Bonded[i]) {
+			if (i < 3)
+				return false; // atoms are still not fully loaded
+			bond[i] = NULL;
+			length[i] = 0;
+			cycle_size[i] = 0;
+			cycle_pos[i] = 0;
+			continue;
+		}
+		bond[i] = static_cast < Bond * > (GetBond (m_Bonded[i]));
+		if (!bond[i]) // not everything has been loaded
+			return false;
+		// search if the bonded atom is a stereocenter
+		if (static_cast < Molecule * > (GetMolecule ())->AtomIsChiral (m_Bonded[i]) ||
+		    bond[i]->IsCyclic ()) {
+			length[i] = cycle_pos[i] = G_MAXUINT; // this will be large enough
+			cycle_size[i] = 0;
+		} else {
+			gcu::Chain *chain = new gcu::Chain (bond[i], this);
+			// find the longuest linear chain and the first attached cycle
+			length[i] = chain->BuildLength (cycle_size + i, cycle_pos + i);
+			// now delete the chain
+			delete chain;
+		}
+		m_Bonded[i]->GetCoords (x + i, y + i);
+		// now sort the atoms with the stereobonds preferably first.
+		send = sorted.end ();
+		for (s = sorted.begin (); s != send; s++) {
+			if (length[*s] > length[i] ||
+			    (length[*s] == length[i] && (cycle_pos[*s] > cycle_pos[i] ||
+			    (cycle_pos[*s] == cycle_pos[i] && (cycle_size[*s] < cycle_size[i] ||
+			    (cycle_size[*s] == cycle_size[i] && (m_Bonded[*s]->GetZ () > m_Bonded[i]->GetZ () || m_Bonded[*s]->GetZ () == 6)))))))
+				break;
+		}
+		sorted.insert (s, i);
+	}
+	unsigned n1, n2;
+	s = sorted.begin ();
+	n1 = *s;
+	s++;
+	n2 = *s;
+	// check if the bond to the n1th atom is properly placed
+	// FIXME
+
+	// parity is evaluated using a determinant as explained in CML reference
+	// assume the third atom is at z=1 and evaluate the determinant
+	// | x0 x1 x2 x3 |
+	// | y0 y1 y2 y3 |
+	// | 0  0  0  1  |
+	// | 1  1  1  1  |
+	// this determinant is the same as (substracting the third column from the
+	// two first):
+	// | x0-x2 x1-x2 x2 x3 |
+	// | y0-y2 y1-y2 y2 y3 |
+	// | 0     0     0  1  |
+	// | 0     0     1  1  |
+	// the determinant is then equal to:
+	// (y0-y2)*(x1-x2)-(y1-y2)*(x0-x2)
+	double invert;
+	if (n1 != 3) {
+		x[n1] = x[3];
+		y[n1] = y[3];
+		invert = -1.;
+	} else
+		invert = 1.;
+	double d = ((y[0] - y[2]) * (x[1] - x[2]) - (y[1] - y[2]) * (x[0] - x[2])) * invert;
+	// for now setting the last bond as stereochemical, clearly bad
+	// first ensure that this is the bond start
+	if (bond[n1]->GetAtom (0) != this)
+		bond[n1]->Revert ();
+	// now, set the correct stereochemistry
+	bond[n1]->SetType ((d > 0)? UpBondType: DownBondType);
+	if (length[n2] == length[n1]) {
+		double a1, a2;
+		a1 = bond[n1]->GetAngle2D (this);
+		a2 = bond[n2]->GetAngle2D (this);
+		a1 -= a2;
+		if (a1 > 360.)
+			a1 -= 360;
+		else if (a1 < 0)
+			a1 += 360.;
+		if (a1 > 180.)
+			a1 = 360. - a1;
+		if (a1 < 90) {
+			if (bond[n2]->GetAtom (0) != this)
+				bond[n2]->Revert ();
+			bond[n2]->SetType ((d > 0)? DownBondType: UpBondType);
+		}
+	}
+	return true;
+}
+
+bool Atom::HasStereoBond () const
+{
+	std::map < gcu::Atom *, gcu::Bond * >::const_iterator i, end = m_Bonds.end ();
+	for (i = m_Bonds.begin (); i != end; i++)
+		switch (static_cast < Bond * > ((*i).second)->GetType ()) {
+		case UpBondType:
+		case DownBondType:
+		case UndeterminedBondType:
+			return true;
+		default:
+			break;
+		}
+	return false;
+}
+
+Bond *Atom::GetNewmanBond () const
+{
+	std::map < gcu::Atom *, gcu::Bond * >::const_iterator i, end = m_Bonds.end ();
+	for (i = m_Bonds.begin (); i != end; i++)
+		if (static_cast < Bond * > ((*i).second)->GetType () == NewmanBondType)
+			return static_cast < Bond * > ((*i).second);
+	return NULL;
+}
+
 }	//	namespace gcp
