@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
- * operation.cc 
+ * operation.cc
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -41,7 +41,7 @@ Operation::Operation (gcp::Document* pDoc, unsigned long ID):
 
 Operation::~Operation ()
 {
-	if (m_Nodes) 
+	if (m_Nodes)
 		delete[] m_Nodes;
 }
 
@@ -77,24 +77,26 @@ void Operation::AddNode (xmlNodePtr node, unsigned type)
 		xmlAddChild (m_Nodes[type], node);
 }
 
- AddOperation:: AddOperation (gcp::Document* pDoc, unsigned long ID):  Operation(pDoc, ID)
+AddOperation::AddOperation (gcp::Document* pDoc, unsigned long ID):  Operation(pDoc, ID)
 {
 	m_Nodes = new xmlNodePtr[1];
 	*m_Nodes = xmlNewDocNode (pXmlDoc, NULL, (const xmlChar*) "add", NULL);
 }
 
- AddOperation::~ AddOperation ()
+AddOperation::~AddOperation ()
 {
-	if (*m_Nodes)
+	if (*m_Nodes) {
+		xmlUnlinkNode (*m_Nodes);
 		xmlFreeNode (*m_Nodes);
+	}
 }
 
-void  AddOperation::Undo ()
+void AddOperation::Undo ()
 {
 	Delete ();
 }
 
-void  AddOperation::Redo ()
+void AddOperation::Redo ()
 {
 	Add ();
 }
@@ -107,8 +109,10 @@ DeleteOperation::DeleteOperation (gcp::Document* pDoc, unsigned long ID): Operat
 
 DeleteOperation::~DeleteOperation ()
 {
-	if (*m_Nodes)
+	if (*m_Nodes) {
+		xmlUnlinkNode (*m_Nodes);
 		xmlFreeNode (*m_Nodes);
+	}
 }
 
 void DeleteOperation::Undo ()
@@ -132,10 +136,14 @@ ModifyOperation::~ModifyOperation ()
 {
 	if (!m_Nodes)
 		return;
-	if (m_Nodes[0])
+	if (m_Nodes[0]) {
+		xmlUnlinkNode (m_Nodes[0]);
 		xmlFreeNode (m_Nodes[0]);
-	if (m_Nodes[1])
+	}
+	if (m_Nodes[1]) {
+		xmlUnlinkNode (m_Nodes[1]);
 		xmlFreeNode (m_Nodes[1]);
+	}
 }
 
 void ModifyOperation::Undo ()
