@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
- * reaction.cc 
+ * reaction.cc
  *
- * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2012 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -32,6 +32,7 @@
 #include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
+#include <gcugtk/ui-manager.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
 #include <cstring>
@@ -41,7 +42,7 @@ using namespace std;
 
 namespace gcp {
 
-Reaction::Reaction (): Object (ReactionType)
+Reaction::Reaction (): Scheme (ReactionType)
 {
 	SetId ("rxn1");
 }
@@ -63,153 +64,135 @@ Reaction::~Reaction ()
 			arrow->SetParent (GetParent ());
 			if (pOp)
 				pOp->AddObject (arrow, 1);
-			
+
 		} else
 			delete pObj;
 	}
 }
 
-bool Reaction::Build (list<Object*>& Children) throw (invalid_argument)
+bool Reaction::Build (std::set < Object * > const &Children) throw (invalid_argument)
 {
 	Document *pDoc = (Document*) GetDocument ();
 	Theme *pTheme = pDoc->GetTheme ();
 	WidgetData  *pData= reinterpret_cast<WidgetData *> (g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data"));
 	map<Object*, gccv::Rect> Objects;
-	list<Object*>::iterator i, end = Children.end ();
-	list<ReactionArrow*> Arrows;
-	list<Object*> Others;
-	map<double, Object*> Left, Right;
-	double x0, y0, x1, y1, x, y, xpos, ypos, l;
-	bool horiz = true;;
-	gccv::Rect *rect, srect;
-	for (i = Children.begin (); i != end; i++) {
+	list < Arrow * >::iterator a, a1, aend;
+	set < Object * >::iterator i, iend = Children.end ();
+	list < Arrow * > Arrows;
+	set < Object * > Others;
+	map < double, Object * > Cur, Left, Right;
+	double x0, y0, x1, y1, x, xpos;
+	double zf = pTheme->GetZoomFactor ();
+	gccv::Rect *rect;
+	for (i = Children.begin (); i != iend; i++) {
 		// It might be better to use the objects coordinates there
 		pData->GetObjectBounds (*i, &Objects[*i]);
 		// Search arrows
 		if ((*i)->GetType() == ReactionArrowType)
 			Arrows.push_front ((ReactionArrow*) (*i));
-		else if ((*i)->GetType() == MoleculeType || (*i)->GetType() == MechanismStepType)
-			Others.push_front (*i);
+		else if ((*i)->GetType () == MoleculeType || (*i)->GetType () == MechanismStepType
+		         || (*i)->GetType () == TextType)
+			Others.insert (*i);
 		else return false;
 	}
-	if (Arrows.size () == 1) {
-	// FIXME: only simple reactions schemes with one arrow are supported in this version
-		ReactionArrow *arrow = Arrows.front ();
-		AddChild (arrow);
-		arrow->GetCoords (&x0, &y0, &x1, &y1);
-		//x0 and y0 should be the center of the arrow, not the beginning, so we must transform them
-		x0 = (x0 + x1) / 2;
-		y0 = (y0 + y1) / 2;
-		// x1, y1 will now be the coordinates of a normalized vector:
-		x1 -= x0;
-		y1 -= y0;
-		x0 *= pTheme->GetZoomFactor ();
-		y0 *= pTheme->GetZoomFactor ();
-		l = sqrt (x1 * x1 + y1 * y1);
-		x1 /= l;
-		y1 /= l;
-		// Now, group objects depending of their position relative to the arrow
-		// FIXME: objects above or below an arrow are not supported.
-		end = Others.end ();
-		for (i = Others.begin (); i != end; i++) {
+	/* sort objects according their position relative to each arrow, using string, with
+	 * one char per arrow, 't' means on tail side, 'h', header side, and 'o' other.
+	 * If we support arrows with several tails or heads, just add a figure
+	 * (there should not be more than 10 heads or tails! */
+	map < string, set < Object * > > steps;
+	iend = Others.end ();
+	aend = Arrows.end ();
+	for (i = Others.begin (); i != iend; i++) {
+		string s;
+		gccv::Rect rect = Objects[*i];
+		double x = (rect.x0 + rect.x1) / 2. / zf, y = (rect.y0 + rect.y1) / 2. / zf;
+		for (a = Arrows.begin (); a != aend; a++)
+			s += static_cast < ReactionArrow * > (*a)->GetSymbolicPosition (x, y);
+		steps[s].insert (*i);
+	}
+	map < string, ReactionStep * > rsteps;
+	map < string, set < Object * > >::iterator istep, step_end = steps.end ();
+	set < ReactionStep * > unused_steps;
+	ReactionStep *step;
+	for (istep = steps.begin (); istep != step_end; istep++) {
+		set < Object * > &cur_set = (*istep).second;
+		set < Object * >::iterator i, end = cur_set.end ();
+		for (i = cur_set.begin (); i != end; i++) {
 			rect = &Objects[*i];
 			xpos = x = (rect->x0 + rect->x1) / 2;
-			y = (rect->y0 + rect->y1) / 2;
-			// search the direction from the center of the arrow to the center of the object
-			x -= x0;
-			y -= y0;
-			l = sqrt (x * x + y * y);
-			x /= l;
-			y /= l;
-			// scalar product:
-			l = x * x1 + y * y1;
-			if (l > 0.71) {
-				while (Right[xpos] != NULL)
-					xpos += 1e-5;
-				Right[xpos] = *i;
-			}
-			else if (l < -0.71) {
-				while (Left[xpos] != NULL)
-					xpos += 1e-5;
-				Left[xpos] = *i;
-			}
-			else //Object too far from the arrow direction
-				throw  invalid_argument(_("Error could not build a reaction\nfrom the selected objects."));
+			while (Cur[xpos] != NULL)
+				xpos += 1e-5;
+			Cur[xpos] = *i;
 		}
-		// We have one or two sets of objects. We must transform them in reaction steps
-		ReactionStep *step;
-		l= 0.;
-		if (Left.size ()) {
-			step = new ReactionStep (this, Left, Objects);
-			// Link fisrt step to the arrow
-			arrow->SetStartStep (step);
-			step->AddArrow (arrow);
-			// Move the arrow to its new position
-			pData->GetObjectBounds (step, &srect);
-			x0 = (srect.x0 + srect.x1) / 2;
-			y0 = step->GetYAlign () * pTheme->GetZoomFactor ();
-			x = srect.x1 - x0;
-			y = srect.y1 - y0;
-			if ((fabs (x1) > 1e-5) && (fabs (y1) > 1e-5))
-				horiz = (fabs (x1) > fabs (y1));
-			else if (fabs (x1) > 1e-5)
-				horiz = true;
-			else
-				horiz = false;
-			if (horiz) {
-				l = x + pTheme->GetArrowPadding ();
-				if (x1 < 0)
-					l = -l;
-				x0 += l;
-				y0 += l * y1 / x1;
+		step = new ReactionStep (this, Cur, Objects);
+		rsteps[(*istep).first] = step;
+		unused_steps.insert (step);
+		Cur.clear ();
+	}
+	// now we must find the relations between the various steps and the arrows
+	// for each arrow, determine the position of head and tail relative to each other arrow
+	for (a = Arrows.begin (); a != aend; a++) {
+		AddChild (*a);
+		(*a)->GetCoords (&x0, &y0, &x1, &y1);
+		// add some padding to the arrow bounds
+		double dx, dy, dl;
+		dx = x1 - x0;
+		dy = y1 - y0;
+		dl = sqrt (dx * dx + dy * dy) / pTheme->GetArrowPadding () * pTheme->GetZoomFactor ();
+		dx /= dl;
+		dy /= dl;
+		x0 -= dx;
+		y0 -= dy;
+		x1 += dx;
+		y1 += dy;
+		std::string st, sh;
+		// search step at tail and head
+		for (a1 = Arrows.begin (); a1 != aend; a1++) {
+			if (a == a1) {
+				st += 't';
+				sh += 'h';
 			} else {
-				l = y + pTheme->GetArrowPadding ();
-				if (y1 < 0)
-					l = -l;
-				x0 += l * x1 / y1;
-				y0 += l;
+				st += static_cast < ReactionArrow * > (*a1)->GetSymbolicPosition (x0, y0);
+				sh += static_cast < ReactionArrow * > (*a1)->GetSymbolicPosition (x1, y1);
 			}
-			arrow->GetCoords (&srect.x0, &srect.y0, &srect.x1, &srect.y1);
-			arrow->Move (x0 / pTheme->GetZoomFactor () - srect.x0, y0 / pTheme->GetZoomFactor () - srect.y0);
 		}
-		arrow->GetCoords (&srect.x0, &srect.y0, &srect.x1, &srect.y1);
-		xpos = srect.x1;
-		ypos = srect.y1;
-		// Create second step
-		if (Right.size ()) {
-			step = new ReactionStep (this, Right, Objects);
-			arrow->SetEndStep (step);
-			step->AddArrow (arrow);
-			pData->GetObjectBounds (step, &srect);
-			x0 = (srect.x0 + srect.x1) / 2;
-			y0 = step->GetYAlign () * pTheme->GetZoomFactor ();
-			if (l == 0.) {
-				if ((fabs (x1) > 1e-5) && (fabs (y1) > 1e-5))
-					horiz = (fabs (x1) > fabs (y1));
-				else if (fabs (x1) > 1e-5)
-					horiz = true;
-				else
-					horiz = false;
-			}
-			if (horiz) {
-				x = srect.x1 - x0;
-				l = x + pTheme->GetArrowPadding ();
-				if (x1 < 0)
-					l = -l;
-				x0 -= l;
-				y0 -= l * y1 / x1;
-			} else {
-				y = srect.y1 - y0;
-				l = y + pTheme->GetArrowPadding ();
-				if (y1 < 0)
-					l = -l;
-				x0 -= l * x1 / y1;
-				y0 -= l;
-			}
-			step->Move (xpos - x0 / pTheme->GetZoomFactor (), ypos - y0 / pTheme->GetZoomFactor ());
+		ReactionStep *start = rsteps[st], *end = rsteps[sh];
+		if (start == NULL) {
+			if (Arrows.size () > 1)
+				throw  invalid_argument (_("Error: arrow without a reactant or product."));
+			// FIXME: add a dummy step
+		} else {
+			unused_steps.erase (start);
+			start->AddArrow (*a, end);
+			(*a)->SetStartStep (start);
 		}
-	} else
-		throw  invalid_argument (_("Error could not build a reaction\nfrom the selected objects."));
+		if (end == NULL) {
+			if (Arrows.size () > 1)
+				throw  invalid_argument (_("Error: arrow without a reactant or product."));
+			// FIXME: add a dummy step
+		} else {
+			unused_steps.erase (end);
+			end->AddArrow (*a, start);
+			(*a)->SetEndStep (end);
+		}
+	}
+
+	// check if all steps are valid
+	map < string, ReactionStep * >::iterator rsi, rsiend = rsteps.end ();
+	for (rsi = rsteps.begin (); rsi != rsiend; rsi++) {
+		if ((*rsi).second && !(*rsi).second->Validate ()) {
+			throw  invalid_argument (_("Error: isolated molecules."));
+			// FIXME: select molecules from this step (should be done in ReactionStep::Validate())
+		}
+	}
+
+	// FIXME: remove dummy steps
+	// if two arrows have no reactant between them, insert an empty step if the arrows are
+	// consecutive or return an error otherwise, that is -> -> is ok -> <- or <- -> are forbidden.
+	// FIXME: if we have an isolated arrow, return false
+
+	// now, position the various steps and arrows
+	Align ();
 	return true;
 }
 
@@ -254,7 +237,7 @@ bool Reaction::OnSignal (SignalId Signal, G_GNUC_UNUSED Object *Obj)
 					horiz = true;
 				else
 					horiz = false;
-				step = arrow->GetStartStep ();
+				step = static_cast < ReactionStep * > (arrow->GetStartStep ());
 				if (step) {
 					has_start = true;
 					pData->GetObjectBounds (step, &rect);
@@ -280,7 +263,7 @@ bool Reaction::OnSignal (SignalId Signal, G_GNUC_UNUSED Object *Obj)
 					arrow->Move (xpos, ypos);
 					pView->Update (arrow);
 				}
-				step = arrow->GetEndStep ();
+				step = static_cast < ReactionStep * > (arrow->GetEndStep ());
 				if (step) {
 					pData->GetObjectBounds (step, &rect);
 					x2 = (rect.x0 + rect.x1) / 2;
@@ -374,17 +357,18 @@ static void do_destroy_reaction (void *data)
 	pDoc->FinishOperation ();
 }
 
-bool Reaction::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y)
+bool Reaction::BuildContextualMenu (gcu::UIManager *UIManager, Object *object, double x, double y)
 {
+	GtkUIManager *uim = static_cast < gcugtk::UIManager * > (UIManager)->GetUIManager ();
 	GtkActionGroup *group = gtk_action_group_new ("reaction");
 	GtkAction *action = gtk_action_new ("destroy", _("Destroy the reaction"), NULL, NULL);
 	gtk_action_group_add_action (group, action);
 	g_object_unref (action);
-	gtk_ui_manager_insert_action_group (UIManager, group, 0);
+	gtk_ui_manager_insert_action_group (uim, group, 0);
 	g_object_unref (group);
 	char buf[] = "<ui><popup><menuitem action='destroy'/></popup></ui>";
-	gtk_ui_manager_add_ui_from_string (UIManager, buf, -1, NULL);
-	GtkWidget *w = gtk_ui_manager_get_widget (UIManager, "/popup/destroy");
+	gtk_ui_manager_add_ui_from_string (uim, buf, -1, NULL);
+	GtkWidget *w = gtk_ui_manager_get_widget (uim, "/popup/destroy");
 	g_signal_connect_swapped (w, "activate", G_CALLBACK (do_destroy_reaction), this);
 	Object::BuildContextualMenu (UIManager, object, x, y);
 	return true;
@@ -399,9 +383,9 @@ double Reaction::GetYAlign ()
 	while (pObj) {
 		if (pObj->GetType () == ReactionStepType)
 			if ((new_y = pObj->GetYAlign ()) < y)
-				y = new_y;	
+				y = new_y;
 		pObj = GetNextChild (i);
-	}			
+	}
 	return y;
 }
 
