@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -38,6 +38,7 @@
 #include <gccv/canvas.h>
 #include <gccv/group.h>
 #include <gccv/text.h>
+#include <gcugtk/ui-manager.h>
 #include <gsf/gsf-output-gio.h>
 #include <gsf/gsf-output-memory.h>
 #include <cairo-pdf.h>
@@ -95,7 +96,7 @@ View::View (Document *pDoc, bool Embedded):
 	m_height = 300;
 	m_ActiveRichText = NULL;
 	m_bEmbedded = Embedded;
-	m_UIManager = gtk_ui_manager_new ();
+	m_UIManager = new gcugtk::UIManager (gtk_ui_manager_new ());
 	m_Dragging = false;
 	m_pWidget = NULL;
 	m_CurObject = NULL;
@@ -121,7 +122,7 @@ View::~View ()
 		g_free (m_sSmallFontName);
 	pango_font_description_free (m_PangoFontDesc);
 	pango_font_description_free (m_PangoSmallFontDesc);
-	g_object_unref (m_UIManager);
+	delete m_UIManager;
 	// we don't need to delete the canvas, since destroying the widget does the job.
 }
 
@@ -178,13 +179,19 @@ void View::Update (Object *pObject)
 	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (pObject);
 	if (client)
 		client->UpdateItem ();
+	// update the links, not that this might result in multiple updates of an object
 	// now, add the children
 	map<string, Object*>::iterator i;
 	Object *child = pObject->GetFirstChild (i);
+	// update the links, not that this might result in multiple updates of an object
 	while (child) {
 		Update (child);
 		child = pObject->GetNextChild (i);
 	}
+	// update the links, not that this might result in multiple updates of an object
+	std::set < gcu::Object * >::iterator j;
+	for (child = pObject->GetFirstLink (j); child; child= pObject->GetNextLink (j))
+		Update (child);
 }
 
 double View::GetZoomFactor ()
@@ -209,15 +216,15 @@ void View::UpdateFont ()
 	m_HWidth = (double (rect.width) / 2.0 + rect.x) / PANGO_SCALE;
 	g_object_unref (G_OBJECT (pl));
 }
-	
+
 void View::Remove (Object* pObject)
 {
 	if (!m_pWidget)
 		return;
 	Object* pObj = pObject->GetMolecule ();
 	if (pObj)
-		m_pData->SelectedObjects.remove (pObj);
-	m_pData->SelectedObjects.remove (pObject);
+		m_pData->SelectedObjects.erase (pObj);
+	m_pData->SelectedObjects.erase (pObject);
 	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (pObject);
 	if (client && client->GetItem ())
 		delete client->GetItem ();
@@ -235,7 +242,7 @@ void View::OnDeleteSelection (GtkWidget* w)
 		Object *pObject, *Group;
 		set<string> ModifiedObjects, DirtyObjects;
 		bool modify = false;
-		list<Object *>::iterator i, iend = m_pData->SelectedObjects.end ();
+		set < Object * >::iterator i, iend = m_pData->SelectedObjects.end ();
 		// first search if we are deleting top objects or not
 		for (i = m_pData->SelectedObjects.begin (); i != iend; i++)
 			if ((*i)->GetGroup ()) {
@@ -256,7 +263,7 @@ void View::OnDeleteSelection (GtkWidget* w)
 				Op->AddObject (*i);
 		}
 		while (!m_pData->SelectedObjects.empty ()) {
-			pObject = m_pData->SelectedObjects.front ();
+			pObject = *m_pData->SelectedObjects.begin ();
 			pObject->Lock ();
 			parent = pObject->GetParent ();
 			if (parent != m_pDoc)
@@ -290,7 +297,7 @@ void View::OnDeleteSelection (GtkWidget* w)
 		}
 	}
 	m_pDoc->FinishOperation ();
-	Window *Win = m_pDoc->GetWindow ();
+	Window *Win = static_cast < Window * > (m_pDoc->GetWindow ());
 	if (Win) {
 		Win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", false);
 		Win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", false);
@@ -386,8 +393,8 @@ void View::OnReceive (GtkClipboard* clipboard, GtkSelectionData* selection_data)
 		} else {
 			GtkAdjustment *horiz, *vert;
 			GtkWidget* parent = gtk_widget_get_parent (m_pWidget);
-			horiz = gtk_viewport_get_hadjustment (GTK_VIEWPORT (parent));
-			vert = gtk_viewport_get_vadjustment (GTK_VIEWPORT (parent));
+			horiz = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (parent));
+			vert = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (parent));
 			dx = gtk_adjustment_get_value (horiz) + gtk_adjustment_get_page_size (horiz) / 2.  - (rect.x0 + rect.x1) / 2.;
 			dy = gtk_adjustment_get_value (vert) + gtk_adjustment_get_page_size (vert) / 2.  - (rect.y0 + rect.y1) / 2.;
 		}
@@ -403,8 +410,8 @@ void View::OnReceive (GtkClipboard* clipboard, GtkSelectionData* selection_data)
 		pTool->AddSelection (m_pData);
 	m_pDoc->PopOperation ();
 	Operation* pOp = m_pDoc->GetNewOperation (GCP_ADD_OPERATION);
-	std::list<Object*>::iterator i, end = m_pData->SelectedObjects.end();
-	for (i = m_pData->SelectedObjects.begin(); i != end; i++) 
+	std::set < Object * >::iterator i, end = m_pData->SelectedObjects.end();
+	for (i = m_pData->SelectedObjects.begin(); i != end; i++)
 		pOp->AddObject (*i);
 	m_pDoc->FinishOperation ();
 }
@@ -430,7 +437,7 @@ void View::OnCutSelection (GtkWidget* w, GtkClipboard* clipboard)
 		OnCopySelection (w, clipboard);
 		OnDeleteSelection (w);
 	}
-	Window *Win = m_pDoc->GetWindow ();
+	Window *Win = static_cast < Window * > (m_pDoc->GetWindow ());
 	if (Win) {
 		Win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", false);
 		Win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", false);
@@ -473,31 +480,31 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 	if (pActiveTool->OnKeyPress (event))
 		return true;
 	switch (event->keyval) {
-	case GDK_Delete:
-	case GDK_Clear:
-	case GDK_BackSpace:
+	case GDK_KEY_Delete:
+	case GDK_KEY_Clear:
+	case GDK_KEY_BackSpace:
 		OnDeleteSelection(w);
 		return true;
-	case GDK_Shift_L:
-	case GDK_Shift_R:
+	case GDK_KEY_Shift_L:
+	case GDK_KEY_Shift_R:
 		if (pActiveTool)
 			pActiveTool->OnKeyPressed (GDK_SHIFT_MASK);
 		return true;
-	case GDK_Control_L:
-	case GDK_Control_R:
+	case GDK_KEY_Control_L:
+	case GDK_KEY_Control_R:
 		if (pActiveTool)
 			pActiveTool->OnKeyPressed (GDK_CONTROL_MASK);
 		return true;
-	case GDK_Alt_L:
-	case GDK_Alt_R:
+	case GDK_KEY_Alt_L:
+	case GDK_KEY_Alt_R:
 		if (pActiveTool)
 			pActiveTool->OnKeyPressed (GDK_MOD1_MASK);//FIXME: might be not portable
 		return true;
-	case GDK_ISO_Level3_Shift:
+	case GDK_KEY_ISO_Level3_Shift:
 		if (pActiveTool)
 			pActiveTool->OnKeyPressed (GDK_MOD5_MASK);//FIXME: might be not portable
 		return true;
-	case GDK_Caps_Lock:
+	case GDK_KEY_Caps_Lock:
 		if (pActiveTool) {
 			if (event->state & GDK_LOCK_MASK)
 				pActiveTool->OnKeyReleased (GDK_LOCK_MASK);
@@ -512,82 +519,82 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 			unsigned min_bonds = (m_CurAtom)? m_CurAtom->GetTotalBondsNumber (): 0;
 			int Z = 0;
 			switch (event->keyval) {
-			case GDK_a:
+			case GDK_KEY_a:
 				Z = 13;
 				break;
-			case GDK_b:
+			case GDK_KEY_b:
 				Z = 5;
 				break;
-			case GDK_c:
+			case GDK_KEY_c:
 				Z = 6;
 				break;
-			case GDK_d:
+			case GDK_KEY_d:
 				Z = 11;
 				break;
-			case GDK_e:
+			case GDK_KEY_e:
 				Z = 34;
 				break;
-			case GDK_f:
+			case GDK_KEY_f:
 				Z = 9;
 				break;
-			case GDK_g:
+			case GDK_KEY_g:
 				Z = 32;
 				break;
-			case GDK_h:
+			case GDK_KEY_h:
 				Z = 1;
 				break;
-			case GDK_i:
+			case GDK_KEY_i:
 				Z = 53;
 				break;
-			case GDK_j:
+			case GDK_KEY_j:
 				Z = 22;
 				break;
-			case GDK_k:
+			case GDK_KEY_k:
 				Z = 19;
 				break;
-			case GDK_l:
+			case GDK_KEY_l:
 				Z = 3;
 				break;
-			case GDK_m:
+			case GDK_KEY_m:
 				Z = 12;
 				break;
-			case GDK_n:
+			case GDK_KEY_n:
 				Z = 7;
 				break;
-			case GDK_o:
+			case GDK_KEY_o:
 				Z = 8;
 				break;
-			case GDK_p:
+			case GDK_KEY_p:
 				Z = 15;
 				break;
-			case GDK_q:
+			case GDK_KEY_q:
 				Z = 14;
 				break;
-			case GDK_r:
+			case GDK_KEY_r:
 				Z = 35;
 				break;
-			case GDK_s:
+			case GDK_KEY_s:
 				Z = 16;
 				break;
-			case GDK_t:
+			case GDK_KEY_t:
 				Z = 78;
 				break;
-			case GDK_u:
+			case GDK_KEY_u:
 				Z = 29;
 				break;
-			case GDK_v:
+			case GDK_KEY_v:
 				Z = 23;
 				break;
-			case GDK_w:
+			case GDK_KEY_w:
 				Z = 74;
 				break;
-			case GDK_x:
+			case GDK_KEY_x:
 				Z = 17;
 				break;
-			case GDK_y:
+			case GDK_KEY_y:
 				Z = 39;
 				break;
-			case GDK_z:
+			case GDK_KEY_z:
 				Z = 40;
 				break;
 			}
@@ -629,8 +636,8 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 			if (entries.empty ())
 				break;
 			map<string, Element*>::iterator i, end = entries.end ();
-			g_object_unref (m_UIManager);
-			m_UIManager = gtk_ui_manager_new ();
+			delete m_UIManager;
+			m_UIManager = new gcugtk::UIManager (gtk_ui_manager_new ());
 			GtkActionGroup *group = gtk_action_group_new ("element");
 			GtkAction *action;
 			string ui;
@@ -642,11 +649,11 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 				gtk_action_group_add_action (group, action);
 				g_object_unref (action);
 				ui = string ("<ui><popup><menuitem action='") + (*i).second->GetSymbol () + "'/></popup></ui>";
-				gtk_ui_manager_add_ui_from_string (m_UIManager, ui.c_str (), -1, NULL);
+				gtk_ui_manager_add_ui_from_string (m_UIManager->GetUIManager (), ui.c_str (), -1, NULL);
 			}
-			gtk_ui_manager_insert_action_group (m_UIManager, group, 0);
+			gtk_ui_manager_insert_action_group (m_UIManager->GetUIManager (), group, 0);
 			g_object_unref (group);
-			GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager, "/popup");
+			GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager->GetUIManager (), "/popup");
 			gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, NULL, 3,  gtk_get_current_event_time ());
 			break;
 		}
@@ -661,18 +668,18 @@ bool View::OnKeyRelease (G_GNUC_UNUSED GtkWidget* w, GdkEventKey* event)
 	if (pActiveTool->OnKeyRelease (event))
 		return true;
 	switch(event->keyval) {
-	case GDK_Shift_L:
-	case GDK_Shift_R:
+	case GDK_KEY_Shift_L:
+	case GDK_KEY_Shift_R:
 		if (pActiveTool)
 			pActiveTool->OnKeyReleased (GDK_SHIFT_MASK);
 		return true;
-	case GDK_Control_L:
-	case GDK_Control_R:
+	case GDK_KEY_Control_L:
+	case GDK_KEY_Control_R:
 		if (pActiveTool)
 			pActiveTool->OnKeyReleased (GDK_CONTROL_MASK);
 		return true;
-	case GDK_Alt_L:
-	case GDK_Alt_R:
+	case GDK_KEY_Alt_L:
+	case GDK_KEY_Alt_R:
 		if (pActiveTool)
 			pActiveTool->OnKeyReleased (GDK_MOD1_MASK);//FIXME: might be not portable
 		return true;
@@ -680,7 +687,7 @@ bool View::OnKeyRelease (G_GNUC_UNUSED GtkWidget* w, GdkEventKey* event)
 		// Seems that when releasing the AltGr key, the keyval is 0 !!!
 		if (!(event->state & GDK_MOD5_MASK))
 			break;
-	case GDK_ISO_Level3_Shift:
+	case GDK_KEY_ISO_Level3_Shift:
 		if (pActiveTool)
 			pActiveTool->OnKeyReleased (GDK_MOD5_MASK);//FIXME: might be not portable
 		return true;
@@ -975,13 +982,13 @@ bool View::OnButtonPressed (gccv::ItemClient *client, unsigned button, double x,
 	}
 	case 3: {
 		bool result;
-		g_object_unref (m_UIManager);
-		m_UIManager = gtk_ui_manager_new ();
+		delete m_UIManager;
+		m_UIManager = new gcugtk::UIManager (gtk_ui_manager_new ());
 		result = pActiveTool->OnRightButtonClicked (this, m_CurObject, x, y, m_UIManager);
 		if (m_CurObject)
 			result |= m_CurObject->BuildContextualMenu (m_UIManager, m_CurObject, x / GetZoomFactor (), y / GetZoomFactor ());
 		if (result) {
-			GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager, "/popup");
+			GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager->GetUIManager (), "/popup");
 			gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, NULL, 3,  gtk_get_current_event_time ());
 			return true;
 		}
@@ -1012,7 +1019,7 @@ bool View::OnButtonReleased (G_GNUC_UNUSED gccv::ItemClient *client, unsigned bu
 bool View::OnDrag (G_GNUC_UNUSED gccv::ItemClient *client, double x, double y, unsigned state)
 {
 	Application *App = m_pDoc->GetApplication ();
-	Tool* pActiveTool = App? App->GetActiveTool (): NULL;	
+	Tool* pActiveTool = App? App->GetActiveTool (): NULL;
 	if (m_pDoc->GetEditable () && pActiveTool && m_Dragging)
 		pActiveTool->OnDrag (x, y, state);
 	return true;
@@ -1030,7 +1037,7 @@ bool View::OnMotion (gccv::ItemClient *client, double x, double y, unsigned stat
 	} else
 		m_CurAtom = NULL;
 	Application *App = m_pDoc->GetApplication ();
-	Tool* pActiveTool = App? App->GetActiveTool (): NULL;	
+	Tool* pActiveTool = App? App->GetActiveTool (): NULL;
 	if (m_pDoc->GetEditable () && pActiveTool)
 		pActiveTool->OnMotion (this, ((m_CurAtom)? m_CurAtom: m_CurObject), x, y, state);
 	return true;
@@ -1039,7 +1046,7 @@ bool View::OnMotion (gccv::ItemClient *client, double x, double y, unsigned stat
 bool View::OnLeaveNotify (unsigned state)
 {
 	Application *App = m_pDoc->GetApplication ();
-	Tool* pActiveTool = App? App->GetActiveTool (): NULL;	
+	Tool* pActiveTool = App? App->GetActiveTool (): NULL;
 	if (m_pDoc->GetEditable () && pActiveTool)
 		pActiveTool->OnLeaveNotify (this, state);
 	return true;
@@ -1049,7 +1056,7 @@ void View::SetSelectionState (Object *object, int state)
 {
 	gccv::ItemClient *client = dynamic_cast <gccv::ItemClient *> (object);
 	if (client)
-		client->SetSelected (state);	
+		client->SetSelected (state);
 	map<string, Object*>::iterator i;
 	Object *child = object->GetFirstChild (i);
 	while (child) {
@@ -1058,4 +1065,17 @@ void View::SetSelectionState (Object *object, int state)
 	}
 }
 
+void View::GetVisibleArea (double &x0, double &y0, double &x1, double &y1)
+{
+	GtkAdjustment *horiz, *vert;
+	GtkWidget* parent = gtk_widget_get_parent (m_pWidget);
+	double zoom = m_pDoc->GetTheme ()->GetZoomFactor ();
+	horiz = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (parent));
+	vert = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (parent));
+	x0 = gtk_adjustment_get_value (horiz) / zoom;
+	x1 = x0 + gtk_adjustment_get_page_size (horiz) / zoom;
+	y0 = gtk_adjustment_get_value (vert) / zoom;
+	y1 = y0 + gtk_adjustment_get_page_size (vert) / zoom;
+}
+
 }	//	namespace gcp
